<?php
/**
 * Exception handler for the Horde_Serialize package.
 *
 * Copyright 2010-2016 Horde LLC (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (LGPL). If you
 * did not receive this file, see http://www.horde.org/licenses/lgpl21.
 *
 * @author   Michael Slusarz <slusarz@horde.org>
 * @category Horde
 * @package  Serialize
 */
class Horde_Serialize_Exception extends Horde_Exception_Wrapped
{
}
